import ChristisRequestor.group as CR_group
import ChristisRequestor.user as CR_user

def get_users_email(group):
    k8sGroup = group
    answerMsg = CR_group.get_group_members(groupCN=k8sGroup)
    if ('ErrorCode' in answerMsg):
        return answerMsg
    members = answerMsg['Response']
    membersEmail = {}
    for member in members:
       answerEmail = CR_user.get_user_email(userDN=member)
       if( type(answerEmail)==dict and 'ErrorCode' in answerEmail):
           membersEmail[member]="404"
           continue
       membersEmail[member]=answerEmail['Response']
    return membersEmail

def get_user_email_byDN(userDN):

    answerMsg = CR_user.get_user_email(userDN=userDN)

    if('ErrorCode' in answerMsg):
        return None
    email = answerMsg['Response']
    return email


import typer
import christis.cmd_config.generate as generate
import christis.cmd_config.view as view

app = typer.Typer()

app.add_typer(generate.app,name="generate",help="Christis components configuration generators")
app.add_typer(view.app,name="view",help="View Christis components configurations")


import typer
import christis.cmd_database.view as db_view
from typing import Optional


app = typer.Typer()

@app.command()
def stage(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
         jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")):
    """
    View users in the stage table
    """

    db_view.stage(verbose=verbose,all=all,jsson=jsson)

@app.command()
def main(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
        jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")
):
    """
    View users in the main table
    """
    db_view.main(verbose=verbose,all=all,jsson=jsson)

@app.command()
def isolated(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
         jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")):
    """
    View users in the isolated users table
    """

    db_view.isolated(verbose=verbose,all=all,jsson=jsson)


import typer
from ChristisMongo.table.get import *  
import os , json
from tabulate import tabulate
from typing import Optional

app = typer.Typer()

@app.command()
def stage(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
         jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")):
    """
    View Stage table
    """

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    answerMsg = get_all_entries(database_name=k8sGroup,table_name="stage")

    if('ErrorCode' in answerMsg):

        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)
    
    stageUser = list(answerMsg['Enteries'])

    if (jsson):
        typer.echo(json.dumps(stageUser))
        typer.Exit(code=0)
        return

    if ( not all and verbose == 0):
        simpleView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            simpleView.append(tempList)
        typer.echo(tabulate(simpleView,tablefmt="pretty",headers=["Emails"]))
    
    if (not all and verbose == 1):

        normalView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            normalView.append(tempList)
        typer.echo(tabulate(normalView,tablefmt="pretty",headers=["Emails","User CN"]))

    if ( not all and verbose == 2):

        extendedView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            tempList.append(user['user_dn'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["Emails","User CN","User DN"]))
    
    if ( all or verbose > 2):
        extendedView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['_id'])
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            tempList.append(user['user_dn'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["ID","Emails","User CN","User DN"]))

@app.command()
def user(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
         jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")):
    """
    View User table
    """

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    answerMsg = get_all_entries(database_name=k8sGroup,table_name="user")

    if('ErrorCode' in answerMsg):

        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)
    
    users = list(answerMsg['Enteries'])

    if (jsson):
        typer.echo(json.dumps(users))
        typer.Exit(code=0)
        return

    if ( not all and verbose == 0):
        simpleView = []
        for user in users:
            tempList = []
            tempList.append(user['email'])
            simpleView.append(tempList)
        typer.echo(tabulate(simpleView,tablefmt="pretty",headers=["Emails"]))
    
    if (not all and verbose == 1):

        normalView = []
        for user in users:
            tempList = []
            tempList.append(user['email'])
            tempList.append(user['user_dn'])
            normalView.append(tempList)
        typer.echo(tabulate(normalView,tablefmt="pretty",headers=["Emails","User DN"]))

    if ( all or verbose == 2):

        extendedView = []
        for user in users:
            tempList = []
            tempList.append(user['_id'])
            tempList.append(user['email'])
            tempList.append(user['user_dn'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["ID","Emails","User DN"]))

@app.command()
def main(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
        jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")
):
    """
    View Main table
    """
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    answerMsg = get_all_entries(database_name=k8sGroup,table_name="main")

    if('ErrorCode' in answerMsg):

        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)

    roleAssignments = list(answerMsg['Enteries'])
    if (jsson):
        typer.echo(json.dumps(roleAssignments))
        typer.Exit(code=0)
        return
    
    if ( not all and verbose == 0):
        simpleView = []
        for role in roleAssignments:
            tempList = []
            tempList.append(role['user_email'])
            tempList.append(role['user_cn'])
            tempList.append(role['role_name'])
            tempList.append(role['role_version'])            
            tempList.append(role['release_name'])
            simpleView.append(tempList)
        typer.echo(tabulate(simpleView,tablefmt="pretty",headers=["User Email","User CN","Role Name","Role Version","Release Name"]))
    
    if (not all and verbose == 1):

        normalView = []
        for role in roleAssignments:
            tempList = []
            tempList.append(role['user_email'])
            tempList.append(role['user_cn'])
            tempList.append(role['user_dn'])
            tempList.append(role['role_name'])
            tempList.append(role['role_version'])            
            tempList.append(role['release_name'])
            normalView.append(tempList)
        typer.echo(tabulate(normalView,tablefmt="pretty",headers=["User Email","User CN","User DN","Role Name","Role Version","Release Name"]))

    if ( all or verbose == 2):

        extendedView = []
        for role in roleAssignments:
            tempList = []
            tempList.append(role['_id'])
            tempList.append(role['user_id'])
            tempList.append(role['user_email'])
            tempList.append(role['user_cn'])
            tempList.append(role['user_dn'])
            tempList.append(role['role_name'])
            tempList.append(role['role_version'])            
            tempList.append(role['release_name'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["Assignment ID","User ID","User Email","User CN","User DN","Role Name","Role Version","Release Name"]))

@app.command()
def isolated(verbose: int = typer.Option(0,"--verbose","-v",count=True),
         all: bool = typer.Option(False, "--all"),
         jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")):
    """
    View Isolated Users table
    """
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    answerMsg = get_all_entries(database_name=k8sGroup,table_name="isolated_users")

    if('ErrorCode' in answerMsg):

        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)
    
    stageUser = list(answerMsg['Enteries'])

    if (jsson):
        typer.echo(json.dumps(stageUser))
        typer.Exit(code=0)
        return

    if ( not all and verbose == 0):
        simpleView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            simpleView.append(tempList)
        typer.echo(tabulate(simpleView,tablefmt="pretty",headers=["Emails"]))
    
    if (not all and verbose == 1):

        normalView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            normalView.append(tempList)
        typer.echo(tabulate(normalView,tablefmt="pretty",headers=["Emails","User CN"]))

    if ( not all and verbose == 2):

        extendedView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            tempList.append(user['user_dn'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["Emails","User CN","User DN"]))
    
    if ( all or verbose > 2):
        extendedView = []
        for user in stageUser:
            tempList = []
            tempList.append(user['_id'])
            tempList.append(user['email'])
            tempList.append(user['user_cn'])
            tempList.append(user['user_dn'])
            extendedView.append(tempList)
        typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["ID","Emails","User CN","User DN"]))




from christis.engine.core.cn import *
from christis.engine.core.email import get_user_email_byDN
from christis.engine.core.general import get_sha2

def get_user_table_enteries(emails: dict , hashEmails: dict):

    data = []

    for userdn,email in emails.items():
        entry = {}
        entry["email"]= email
        entry["user_dn"] = userdn
        data.append(entry)
    
    for e in data:
        e['_id'] = hashEmails[e['user_dn']]
    
    return data

def get_stage_table_enteries(stageCandidateList: list):

    stageUserList = []
    
    for user in stageCandidateList:
        stageCandidate = {}
        email = get_user_email_byDN(user)
        stageCandidate['_id'] = get_sha2(email)
        stageCandidate['email'] = email
        stageCandidate['user_cn'] = get_user_cn(user)
        stageCandidate['user_dn'] = user
        stageUserList.append(stageCandidate)
    
    return stageUserList
        
    



from pymongo import results
import typer
from typing import List, Optional
import os
from ChristisMongo.table.get import *
from enum import Enum
from ChristisMongo.table.query import *
import subprocess
from pathlib import Path
import yaml , json
from christis.engine.core.general import get_domain_part_from_dn, get_sha2
from ChristisMongo.table.add import *
from ChristisMongo.table.delete import *
from tabulate import tabulate


app = typer.Typer()
exportcmd = typer.Typer()
importcmd = typer.Typer()
app.add_typer(importcmd,name="import",help="import template for bulk role assignment")
app.add_typer(exportcmd,name="export",help="export a template from stage user for bulk role assignment")

class attributeType (str, Enum):
    dn = "dn"
    cn = "cn"
    email = "email"
    #id = "id"

@app.command()
def view(
    user_attr: str = typer.Option(...,'--attribute-value',help='The user attribute value that is used to find user'),
    attr_type : attributeType = typer.Option(...,'--attribute-type',help="The attribute type of user-attribute",case_sensitive=False),
    role: Optional[str] = typer.Option(None,"--role",help="the role name"),
    verbose: int = typer.Option(0,"--verbose","-v",count=True),
    jsson: Optional[bool] = typer.Option(False, "--json",help="Output format in Json")
):

    """
    View the users role assignment 
    """

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    if(role is None):

        query = {"user_{0}".format(attr_type): user_attr}
    else:
        query = {"user_{0}".format(attr_type): user_attr,"role_name":role}

    answerMsg = query_abstract(database_name=k8sGroup,table_name='main',query=query)

    if('ErrorCode' in answerMsg):
        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)
    enteries = list(answerMsg['Enteries'])
    if(len(enteries)==0):
        typer.echo("The user with the attribute value of {0} and attribute type of {1} is not found. No role assignemet!".format(user_attr,attr_type))
        raise typer.Exit(code=1)
    
    if (jsson):
        typer.echo(json.dumps(enteries))
        typer.Exit(code=0)
        return

    if(verbose==0):
        normalView = []
        for entry in enteries:
            tempList = []
            tempList.append(entry['user_email'])
            tempList.append(entry['user_cn'])
            tempList.append(entry['user_dn'])
            tempList.append(entry['role_name'])
            tempList.append(entry['role_version'])
            tempList.append(entry['release_name'])
            normalView.append(tempList)
            typer.echo(tabulate(normalView,tablefmt="pretty",headers=["User Email","User CN","User DN","Role Name","Role Version","Release Name"]))
    if(verbose>=1):
        extendedView = []
        for entry in enteries:
            tempList = []
            tempList.append(entry['_id'])
            tempList.append(entry['user_id'])
            tempList.append(entry['user_email'])
            tempList.append(entry['user_cn'])
            tempList.append(entry['user_dn'])
            tempList.append(entry['role_name'])
            tempList.append(entry['role_version'])
            tempList.append(entry['release_name'])
            extendedView.append(tempList)
            typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["Assignment ID","User ID","User Email","User CN","User DN","Role Name","Role Version","Release Name"]))


@app.command()
def assign (
    user_attr: str = typer.Option(...,'--attribute-value',help='The user attribute value that is used to find and assign role to user'),
    attr_type : attributeType = typer.Option(...,'--attribute-type',help="The attribute type of user-attribute",case_sensitive=False),
    roleList: List[str] = typer.Option(...,'--role',help='The role name and version that should be assigned to user. should be in this format <RoleName>*<RoleVersion>'),
    varFile: Optional[Path] = typer.Option(None,"--override-var-file",help="The variable file that override the default variable file of Role")
):
    """
    Assign Role to the User
    """
    
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']
    user = []
    if(attr_type == 'dn'):
        answerMsg = query_user_by_dn(database_name=k8sGroup,table_name='stage',dn=user_attr)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        user = list(answerMsg['Enteries'])
        if (len(user)==0):
            typer.echo("User with this of DN {0} is not found in the stage table.".format(user_attr),err=True)
            return 1
    
    if(attr_type == 'cn'):
        answerMsg = query_user_by_cn(database_name=k8sGroup,table_name='stage',cn=user_attr)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        user = list(answerMsg['Enteries'])
        if (len(user)==0):
            typer.echo("User with the CN of {0} is not found in the stage table.".format(user_attr),err=True)
            return 1
    
    if(attr_type == 'email'):
        answerMsg = query_user_by_email(database_name=k8sGroup,table_name='stage',email=user_attr)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        user = list(answerMsg['Enteries'])
        if (len(user)==0):
            typer.echo("User with the Email of {0} is not found in the stage table.".format(user_attr),err=True)
            return 1
    
    if(attr_type == 'id'):
        answerMsg = query_user_by_id(database_name=k8sGroup,table_name='stage',id=user_attr)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        user = list(answerMsg['Enteries'])
        if (len(user)==0):
            typer.echo("User with the Id of {0} is not found in the stage table.".format(user_attr),err=True)
            return 1
    
    approvedRole = {}

    # Check VarFile
    if varFile is None:
        varFile = "None"
    else:
        if varFile.is_dir():
            typer.echo("ERROR: The var file is directory should be file!",err=True)
            raise typer.Exit(code=1)
        
        varFileName, varExt = os.path.splitext(varFile)

        if(not(varExt == '.yml' or varExt == '.yaml')):
            typer.echo("ERROR: The var file should be yaml or yml",err=True)
            raise typer.Exit(code=1)
        
        with open(varFile, 'r') as stream:
            try:
                pass
            except yaml.YAMLError as exc:
                typer.echo("ERROR: The var file couldn't be read",err=True)
                raise typer.Exit(code=1)
    

    for role in roleList:

        if "*" not in role:
            typer.echo("Error: Bad role format. It should be in this format <RoleName>*<RoleVersion>")
            raise typer.Exit(code=1)

        roleName = role.split('*')[0]
        roleVersion = role.split("*")[1]

        query = {"role_name" : roleName, "version": roleVersion}

        answerMsg = query_abstract(database_name=k8sGroup,table_name='role',query=query)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        roleQ = list(answerMsg['Enteries'])
        if(len(roleQ)==0):
            typer.echo("The role with the name of {0} and version of {1} is not found! Failed to assign the role.".format(roleName,roleVersion))
            return 1
        if(varFile is None):
            approvedRole[roleName] = {'version':roleVersion,'path':roleQ[0]['location'],'varFile':roleQ[0]['var_file'],'id':roleQ[0]['_id']}
        else:
            approvedRole[roleName] = {'version':roleVersion,'path':roleQ[0]['location'],'varFile':varFile,'id':roleQ[0]['_id']}
        
    #UserDetails
    userCN = user[0]['user_cn']
    userDN = user[0]['user_dn']
    userEmail = user[0]['email']
    userId = user[0]['_id']

    typer.echo("Assigning Role to the user: {1}={0} ...".format(user_attr,attr_type))
    with typer.progressbar(approvedRole.items(),label="Processing") as progress:
        for r in progress:
            roleName = r[0]
            roleVersion = r[1]['version']
            roleId = r[1]['id'] 
            # releaseName = <group>-cn-dc-rolename-roleversion
            userDomainPart = get_domain_part_from_dn(userDN)
            releaseName = "{0}-{1}-{2}-{3}-{4}".format(k8sGroup,userCN.replace(" ", ""),userDomainPart,roleName.replace(" ", ""),roleVersion.replace(" ", "")) # remove the space from the rolename and version and user CN
            CNvariable = 'user_cn={0}'.format(userCN)
            typer.echo("ReleaseName : {0}".format(releaseName))
            # Have to !
            DNvariable = 'user_dn={0}'.format(userDN.replace('=',':').replace(',','-'))
            EmailVariable = 'user_email={0}'.format(userEmail)
            IDvariable = 'user_id={0}'.format(userId)
            GroupVariable = 'user_k8_group={0}'.format(k8sGroup)

            # check the helm release is exist or not
            checkCommand = ['helm','list','-o','json','--filter',releaseName]

            checkOut = subprocess.Popen(checkCommand,stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
            chkStdOut , chkStdErr = checkOut.communicate()

            if(checkOut.returncode == 1):
                typer.echo("ERROR: The Release existance checking faild.")
                typer.echo("\n")
                typer.echo(chkStdOut.decode('utf-8'))
                raise typer.Exit(code=1)
            
            jsonChk = json.loads(chkStdOut.decode('utf-8'))
            if( len(jsonChk) > 0):
                typer.echo("ERROR: A release with the name of ' {0} ' exists that is using the role of ' {1} ' with version of ' {2} '.".format(releaseName,roleName,roleVersion))
                typer.Abort()
                continue

            if(r[1]['varFile']=='None'):
                command = ['helm','install',releaseName,"--set",CNvariable,"--set",DNvariable,"--set",EmailVariable,"--set",IDvariable,"--set",GroupVariable,r[1]['path']]
            else:
                command = ['helm','install',releaseName,"--set",CNvariable,"--set",DNvariable,"--set",EmailVariable,"--set",IDvariable,"--set",GroupVariable,"-f",r[1]['varFile'],r[1]['path']]
            out = subprocess.Popen(command, 
            stdout=subprocess.PIPE, 
            stderr=subprocess.STDOUT)

            stdout,stderr = out.communicate()
            if (out.returncode == 1):
                typer.echo("ERROR: The role assignment is faild.")
                typer.echo("\n")
                typer.echo(stdout.decode('utf-8'))
                raise typer.Exit(code=1)
            else:
                typer.echo("The role {0} with the version of {1} is assigned.".format(roleName,roleVersion))
            
            releaseID = get_sha2(releaseName)

            mainEntry = {"_id":releaseID,"user_id":userId,"user_email":userEmail,"user_cn":userCN,"user_dn":userDN,"role_name":roleName,"role_version":roleVersion,"role_id":roleId,'release_name':releaseName}
            answerMsg = add_entry_one(database_name=k8sGroup,data=mainEntry,table_name='main')
            
            if('ErrorCode' in answerMsg):
                uninstallCommand = ['helm','uninstall',releaseName]
                outUninstall = subprocess.Popen(uninstallCommand, 
                                stdout=subprocess.PIPE, 
                                stderr=subprocess.STDOUT)
                typer.echo("Error! Release will be uinstalled!",err=True)
                if(answerMsg['ErrorCode'] == '701'):        
                    typer.echo("Error: A release with this Id exists on the Main table!",err=True)
                    continue
                else:
                    typer.echo("Error : {0}".format(answerMsg['ErrorMsg']),err=True)
                    raise typer.Exit(code=1)

@exportcmd.command()
def template(
    location: Path = typer.Option(...,"--location",help="The template's absolute path to export")
):
    """
    Export template
    """
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    answerMsg = get_all_entries(database_name=k8sGroup,table_name='stage')

    if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
    user = list(answerMsg['Enteries'])
    if (len(user)==0):
        typer.echo("There is no users in the stage table.",err=True)
        raise typer.Exit(code=1)
    
    template = []

    for u in user:
        userDict = { u['email']: {'role':{'role1':'version','role2':'version'},'variableFile':'location'}}
        template.append(userDict)

    with open(location, 'w') as file:
        documents = yaml.dump(template, file)

    typer.echo("Export Done!")
    typer.Exit(code=0)
                   
@importcmd.command()
def template(
    location: Path = typer.Option(...,"--location",help="The template's absolute path to export")
):  
    """
    Import Template
    """
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    if location.is_dir():
        typer.echo("ERROR: The var file is directory should be file!",err=True)
        raise typer.Exit(code=1)
        
    fileName, fileExt = os.path.splitext(location)
    if(not(fileExt == '.yml' or fileExt == '.yaml')):
        typer.echo("ERROR: The template file should be yaml or yml",err=True)
        raise typer.Exit(code=1)

    with open(location, 'r') as stream:
        try:
            users = yaml.full_load(stream)
        except yaml.YAMLError as exc:
            typer.echo("ERROR: The tempate file couldn't be read: {0}".format(exc),err=True)
            raise typer.Exit(code=1)
    
    with typer.progressbar(users) as progress:
        for user in progress:
            for email in user:
                
                roles = user[email]['role']
                for role,version in  roles.items():
                    typer.echo("\n")
                    typer.echo("*********************************************************")
                    typer.echo("\n")
                    if(user[email]['variableFile'] == 'None'):
                        result = assign(attr_type="email",roleList=["{0}*{1}".format(role,version)],user_attr=email,varFile=None)
                    else:
                        result = assign(attr_type="email",roleList=["{0}*{1}".format(role,version)],user_attr=email,varFile=user[email]['variableFile'])
                    if(result == 1):
                        typer.echo("The role assignmnet for the role of {0} with the version of {1} to the user {2} is failed.".format(role,version,email))

@app.command()
def unassign(
    user_attr: str = typer.Option(...,'--attribute-value',help='The user attribute value that is used to find and unassign role from user'),
    attr_type : attributeType = typer.Option(...,'--attribute-type',help="The attribute type of user-attribute",case_sensitive=False),
    role: str = typer.Option(...,'--role',help='The role name and version that should be unassigned from user. should be in this format <RoleName>*<RoleVersion> . ALL can be used to unassign all roles.')):
    """
    Unassign a Role from user 
    """
    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']

    if (role != "ALL" and "*" not in role):
            typer.echo("Error: Bad role format. It should be in this format <RoleName>*<RoleVersion>")
            raise typer.Exit(code=1)

    roleName = ""
    roleVersion = ""
    if(role == "ALL"):
        query = {"user_{0}".format(attr_type): user_attr}
    else:
        roleName = role.split('*')[0]
        roleVersion = role.split("*")[1]
        query = {"user_{0}".format(attr_type): user_attr,"role_name":roleName,"role_version":roleVersion}

    answerMsg = query_abstract(database_name=k8sGroup,table_name='main',query=query)

    if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
    assignmentEnteries = list(answerMsg['Enteries'])
    if(len(assignmentEnteries)==0):
        if(role != "ALL"):
            typer.echo("The role assignment for user {0}: {1} and role: {2} and version: {3} is not found!".format(attr_type,user_attr,roleName,roleVersion),err=True)
            return 1
        else:
            typer.echo("No role assignment for the user {0}: {1} is found!".format(attr_type,user_attr),err=True)
            return 1
    
    for assignment in assignmentEnteries:

        releaseName = assignment['release_name']
        assignmentID = assignment['_id']
        
        roleName = assignment['role_name']
        roleVersion = assignment['role_version']
        typer.echo("The release name : {0}".format(releaseName))
        deleteQuery = {"_id":assignmentID}
        answerMsg= delete_abstract_one(database_name=k8sGroup,query=deleteQuery,table_name='main')
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        
        uninstallCommand = ['helm','uninstall',releaseName]
        outUninstall = subprocess.Popen(uninstallCommand, 
                            stdout=subprocess.PIPE, 
                            stderr=subprocess.STDOUT)

        stdout,stderr = outUninstall.communicate()
        if (outUninstall.returncode == 1):
            typer.echo("ERROR: The role unassigning is faild.")
            typer.echo("\n")
            typer.echo(stdout.decode('utf-8'))
            # return back the enteries to main table after failing uninstalling!
            add_entry_one(data=assignment,database_name=k8sGroup,table_name='main')
            raise typer.Exit(code=1)
        else:
            typer.echo("The role {0} with the version of {1} is unassigned.".format(roleName,roleVersion))

        

@app.command()
def purge(
    user_attr: str = typer.Option(...,'--attribute-value',help='The user attribute value that is used to find and purge role of isolated user'),
    attr_type : attributeType = typer.Option(...,'--attribute-type',help="The attribute type of user-attribute",case_sensitive=False)):
    """
    Unassign all roles of isoalted user 
    """
    if(attr_type == attributeType.email):
        query = {"email":user_attr}
    else:    
        query = {"user_{0}".format(attr_type): user_attr}

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']


    answerMsg = query_abstract(database_name=k8sGroup,table_name='isolated_users',query=query)

    if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
    assignmentEnteries = list(answerMsg['Enteries'])
    if(len(assignmentEnteries)==0):
        
        typer.echo("There is no user with this information {0} : {1} in the Isolated table!".format(attr_type,user_attr),err=True)
        return 1
    
    unassign(user_attr=user_attr,attr_type=attr_type,role='ALL')
    




import typer
import os
from christis.engine.core.email import *
from christis.engine.core.general import *
from christis.engine.process.generate_enteries import *
from christis.engine.insert.user_table import *
from ChristisMongo.core.collection import get_collection
from ChristisMongo.table.add import *
import ChristisMongo.table.query as CM_query 
import ChristisMongo.table.delete as CM_delete
from christis.engine.process.check import check_ldap_deleted_user
import christis.cmd_database.view as view

app = typer.Typer()

app.add_typer(view.app,name="view",help="Commands for viewing tables like stage,main,...")

@app.command()
def sync():

    """
    Sync users from K8sGroup in your Active Directory to your Christis Database
    """

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']
    userEmails = get_users_email(k8sGroup)
    finalUserEmail = {}
    for user,email in userEmails.items():
        if (email=='404'):
            typer.echo("Error! The Email is not set for user: {0}. The user will be removed from update state!".format(user),err=True)
            continue
        finalUserEmail[user]=email
    
    emailsHash = {}
    for user,email in finalUserEmail.items():
        emailsHash[user] = get_sha2(email)
    # These users can be added to pre_user , user and stage
    userTableList = get_user_table_enteries(emails=finalUserEmail,hashEmails=emailsHash)

    # These list will be used to check user deletion
    preUserList = add_2_pre_user_table(userList=userTableList,group=k8sGroup)

    # Add these users to the user table , if the use can be added to user table it can be added to stage too!
    userCandidateForStage = add_2_user_table(userList=userTableList,group=k8sGroup)

    # These users should be deleted from stage,main,user because they are remove from K8s group in LDAP
    userCandidateToDelete = check_ldap_deleted_user(k8group=k8sGroup)

    if(len(userCandidateToDelete)>0):
        typer.echo("The roles and resources of following users will be deleted : ")
        i = 1
        for deletedUser in userCandidateToDelete:
            deletedUserEmail = deletedUser['email']
            typer.echo("User {0} >>>  Email : {1} | DN : {2} ".format(i,deletedUserEmail,deletedUser['user_dn']))
            i = i + 1
        # TODO Send Delete Operatin!!!! 
        # TODO : Should be deleted form User table and stage(if it exists on it) and from main!
        # The user first will be moved to the isolated users table and then it will be deleted form stage and user table
            answerMsg = CM_query.query_user_by_email(database_name=k8sGroup,table_name='stage',email=deletedUserEmail)

            if('ErrorCode' in answerMsg):
                typer.echo("Database Error!",err=True)
                typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                raise typer.Exit(code=1)
            user = list(answerMsg['Enteries'])
            if (len(user)==0):
                typer.echo("User with this Email {0} is not found in the stage table.".format(deletedUserEmail),err=True)
                return 1
            
            answerMsg = add_entry_one(database_name=k8sGroup,table_name="isolated_users",data=user[0])
            if('ErrorCode' in answerMsg):
                if(answerMsg['ErrorCode'] == '701'):        
                    typer.echo("Error: An user {0} exists on the Isolated Users table!".format(deletedUserEmail),err=True)
                    continue
                else:
                    typer.echo("The user {0} can't be moved to Isolated User Table.".format(deletedUserEmail),err=True)
                    typer.echo("Database Error!",err=True)
                    typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                    raise typer.Exit(code=1)
            
            answerMsg = CM_delete.delete_user_by_email(database_name=k8sGroup,table_name='stage',email=deletedUserEmail)
            if('ErrorCode' in answerMsg):
                typer.echo("The user {0} can't be removed from Stage Table.".format(deletedUserEmail),err=True)
                typer.echo("Database Error!",err=True)
                typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                raise typer.Exit(code=1)
            typer.echo("The User {0} is moved to Isolated table and you can't assign any role to it.".format(deletedUserEmail))
            answerMsg = CM_delete.delete_user_by_email(database_name=k8sGroup,table_name='user',email=deletedUserEmail)
            if('ErrorCode' in answerMsg):
                typer.echo("The user {0} can't be removed from User Table.".format(deletedUserEmail),err=True)
                typer.echo("Database Error!",err=True)
                typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                raise typer.Exit(code=1)
    else:
        typer.echo("No user was deleted from {0} group. No candidate to delete!".format(k8sGroup))

    # These data and users will be added to the stage table
    StageUserList = get_stage_table_enteries(stageCandidateList=userCandidateForStage)
    if(len(StageUserList)>0):
        typer.echo("The following users will be added to the Stage : ")
        i = 1
        for stageUser in StageUserList:
            stageUserEmail = stageUser['email']
            typer.echo("User {0} >>>  Email : {1} | DN : {2} ".format(i,stageUser['email'],stageUser['user_dn']))
            i = i + 1
            # When an isolated user back to k8s group it should be deleted from the isoalted user table
            answerMsg = CM_query.query_user_by_email(database_name=k8sGroup,table_name='isolated_users',email=stageUserEmail)
            if('ErrorCode' in answerMsg):
                typer.echo("Database Error!",err=True)
                typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                raise typer.Exit(code=1)
            user = list(answerMsg['Enteries'])
            if (len(user) > 0):
                answerMsg = CM_delete.delete_user_by_email(database_name=k8sGroup,table_name='isolated_users',email=stageUserEmail)
                if('ErrorCode' in answerMsg):
                    typer.echo("Database Error!",err=True)
                    typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
                    raise typer.Exit(code=1)
            else:
                continue
        # Add enteries to the STAGE
        add_entry_multiple(table_name="stage",database_name=k8sGroup,data=StageUserList)
    else:
        typer.echo("The Stage is up to date. No user will be added to.")

    #delete the pre_user table at the end
    preUserTable = get_collection(collection_name="pre_user",db_name=k8sGroup)
    preUserTable.drop()




import ChristisMongo.table.get as CM_get
import ChristisMongo.table.query as CM_query

# The method check users of User and PreUser table to find which user was deleted from LDAP
def check_ldap_deleted_user(k8group):

    users = CM_get.get_all_entries(database_name=k8group,table_name="user")
    candidateToDelete = []
    for user in users['Enteries']:
        result = CM_query.query_user_by_id(database_name=k8group,table_name='pre_user',id=user['_id'])
        if ('ErrorCode' in result):
            continue
        if(len(list(result['Enteries']))==0):
            candidateToDelete.append(user)
    return candidateToDelete





from click.termui import confirm
import typer
import yaml
from pathlib import Path
from christis.vars import *
import os


app = typer.Typer()

@app.command()
def cli(
    api_address: str = typer.Option(...,"--api-address",envvar="CHRISTIS_API_ADDRESS",help="Christis API Server Address"),
    api_port: str = typer.Option(...,"--api-port",envvar="CHRISTIS_API_PORT",help="Christis API Server Port"),
    k8s_group: str = typer.Option(...,"--k8s-group",envvar="CHRISTIS_K8s_LDAP_GROUP",help="The group that its user should access to K8s cluster"),
    ):

    """
    Generate a ChristisCLI configuration file in the ~/.christisCLI location
    """

    if not Path(get_mongo_configuration_location()).is_file():
        typer.echo(f"The Database config file is not found. Please generate it via CLI and try again.",err=True)
        raise typer.Abort()
    config = {}
    config['ChristisCLI'] = {}
    CLI_Config_Location = get_christis_config_location()
    configFileLocation = "{0}{1}".format(CLI_Config_Location,"cli.yaml")
    if Path(configFileLocation).is_file():
        overwrite = typer.confirm("The CLI configuration exists, Do you want to generate it again?")
        if not overwrite:
            typer.echo("Not Generating New Configuration File")
            raise typer.Abort()
    typer.echo("Generate New Config File")
    config['ChristisCLI']["CHRISTIS_API_ADDRESS"] = api_address
    config['ChristisCLI']["CHRISTIS_API_PORT"] = api_port
    config['ChristisCLI']["CHRISTIS_K8s_LDAP_GROUP"] = k8s_group
    config['ChristisCLI']["CHRISTIS_MONGO_CONFIG_FILE"] = get_mongo_configuration_location()

    if not os.path.exists(CLI_Config_Location):
        os.makedirs(CLI_Config_Location)
    try:
        
        with open(configFileLocation,'w') as file:
            configFile = yaml.dump(config,file)
    except Exception as e:
        typer.echo(e,err=True)
        raise typer.Abort()     


@app.command()
def database(
    mongo_address: str = typer.Option(...,"--mongodb-address",envvar="MONGODB_ADDRESS",help="The address of MongoDB"),
    mongo_user: str = typer.Option(...,"--mongodb-user",envvar="MONGODB_USER",help="The username that access to database"),
    mongo_password: str = typer.Option(...,"--mongodb-password",help="The password of user who access the database"),
):
    """
    Generate MongoDB configuration file that need to access database in the ~/.christisCLI location
    """
    config = {}
    config["MongoDB"] = {}

    CLI_Config_Location = get_christis_config_location()
    configFileLocation = "{0}{1}".format(CLI_Config_Location,"mongo.yaml")
    if Path(configFileLocation).is_file():
        overwrite = typer.confirm("The MongoDB configuration exists, Do you want to generate it again?")
        if not overwrite:
            typer.echo("Not Generating New Configuration File")
            raise typer.Abort()
    typer.echo("Generate New Config File")
    config["MongoDB"]["mongo_address"] = mongo_address
    config["MongoDB"]["mongo_user"] = mongo_user
    config["MongoDB"]["mongo_password"] = mongo_password

    if not os.path.exists(CLI_Config_Location):
        os.makedirs(CLI_Config_Location)
    try:
        with open(configFileLocation,'w') as file:
            configFile = yaml.dump(config,file)
    except Exception as e:
        typer.echo(e,err=True)
        raise typer.Abort()

@app.command()
def christis_API(
    ldap_server: str = typer.Option(...,"--ldap-server-address",help="The LDAP server address. Should be in this format LDAP://<Address>"),
    ldap_user_connector: str = typer.Option(...,"--ldap-user",help="The User DN that can access to server to query like  CN=Administrator,CN=Users,DC=cloudarmin,DC=local"),
    ldap_password_connector: str = typer.Option(...,"--ldap-password",help="The password of user who access to server"),
    ldap_base_dn: str = typer.Option(...,"--ldap-base-dn",help="The base DN of you LDAP domain like dc=cloudarmin,dc=local"),
    ldap_k8s_group: str = typer.Option(...,"--ldap-k8s-group",help="The cn of group that its users should access the K8s cluster")):

    """
    Generate ChristisCLI configuration file that will be used by ChristisAPI Docker container in the ~/.christisCLI location
    """
    config = {}
    config["ldap"] = {}

    CLI_Config_Location = get_christis_config_location()
    configFileLocation = "{0}{1}".format(CLI_Config_Location,"ldap.yaml")
    if Path(configFileLocation).is_file():
        overwrite = typer.confirm("The LDAP configuration exists, Do you want to generate it again?")
        if not overwrite:
            typer.echo("Not Generating New Configuration File")
            raise typer.Abort()
    typer.echo("Generate New Config File")

    config["ldap"]["ldap_server"] = ldap_server
    config["ldap"]["ldap_user_connector"] = ldap_user_connector
    config["ldap"]["ldap_password_connector"] = ldap_password_connector
    config["ldap"]["ldap_base_dn"] = ldap_base_dn
    config["ldap"]["ldap_k8s_group"] = ldap_k8s_group

    if not os.path.exists(CLI_Config_Location):
        os.makedirs(CLI_Config_Location)
    try:
        with open(configFileLocation,'w') as file:
            configFile = yaml.dump(config,file)
    except Exception as e:
        typer.echo(e,err=True)
        raise typer.Abort()


import yaml
import os
import sys

def get_configuration(configPath):
    if os.path.exists(configPath):
        try:
            with open(configPath) as file:
                configuration = {}
                configuration = yaml.load(file, Loader=yaml.FullLoader)
        except IOError as e:
            Error = {"ErrorCode":"805","ErrorMsg":"I/O error({0}): {1}".format(e.errno, e.strerror)}
        except:
            Error = {"ErrorCode":"805","ErrorMsg":sys.exc_info()[1]}
            return Error
        else:       
            return configuration
    else:
        Error = {"ErrorCode":"804","ErrorMsg":"Config File not found"}
        return Error


import typer
from pathlib import Path
from typing import Optional
import subprocess
import os 
from shutil import which
from christis.engine.core.general import get_sha2
import yaml
import ChristisMongo.table.add as CM_add
from tabulate import tabulate
from ChristisMongo.table.get import *
import ChristisMongo.table.query as CM_query


app = typer.Typer()

@app.callback()
def main(ctx: typer.Context):

    if(ctx.invoked_subcommand != 'create'):
         return
    
    if (which('helm') == None):
        typer.echo("ERROR: Helm is not found! Please install it.",err=True)
        raise typer.Exit(code=1)

@app.command()
def create(
    name: str = typer.Option(...,"--name",help="The role name"),
    location: Path = typer.Option(...,"--location",help="The location of Helm Chart that specifies your role. Can be in .tgz or folder"),
    description: str = typer.Option(...,"--description",help="A description for your role"),
    version: str = typer.Option(...,"--version",help="The role version"),
    varFile : Optional[Path] = typer.Option(None,"--var-file",help="The variable file that the role needs besides of the variables that are provided by CLI")
):
    """
    Create a Role from Helm Chart and save it in the Database
    """
    if location.is_file():
        filename, file_extension = os.path.splitext(location)

        if (not file_extension == '.tgz'):
            typer.echo("ERROR: The role file is not in proper package format",err=True)
            raise typer.Exit(code=1)
    #TODO : Check varfile exist!?
    if varFile is None:
        varFile = "None"
    else:
        if varFile.is_dir():
            typer.echo("ERROR: The var file is directory should be file!",err=True)
            raise typer.Exit(code=1)
        
        varFileName, varExt = os.path.splitext(varFile)

        if(not(varExt == 'yml' or varExt == 'yaml')):
            typer.echo("ERROR: The var file should be yaml or yml",err=True)
            raise typer.Exit(code=1)
        
        with open(varFile, 'r') as stream:
            try:
                pass
            except yaml.YAMLError as exc:
                typer.echo("ERROR: The var file couldn't be read",err=True)
                raise typer.Exit(code=1)
    
    Lint_out = subprocess.Popen(['helm', 'lint', location], 
           stdout=subprocess.PIPE, 
           stderr=subprocess.STDOUT)
    stdout,stderr = Lint_out.communicate()

    if (Lint_out.returncode == 1):
        typer.echo("ERROR: The role's linting is faild.")
        typer.echo("\n")
        typer.echo(stdout.decode('utf-8'))
        raise typer.Exit(code=1)
    else:
        typer.echo("Role's linting is sucessful.")
    
    roleID = get_sha2("{0}-{1}".format(name,version))
    
    role = {
        "_id": roleID,
        "role_name": name,
        "location": str(location),
        "version" : str(version),
        "var_file" : str(varFile),
        "description" : description
    }

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']
    result = CM_add.add_entry_one(database_name=k8sGroup,table_name='role',data=role)

    if ('ErrorCode' in result):
        if(result['ErrorCode'] == '701'):
            typer.echo("ERROR: The role with these name and version exists. Role can't be added",err=True)
            raise typer.Exit(code=1)
        else:
            typer.echo("ERROR: Error Message : {0}.".format(result['ErrorMsg']),err=True)
            raise typer.Exit(code=1)

@app.command()
def view(
    name: Optional[str] = typer.Option(None,"--name",help="The role name"),
    verbose: int = typer.Option(0,"--verbose","-v",count=True),
):

    """
    View the Roles that are already available
    """

    k8sGroup = os.environ['CHRISTIS_K8s_LDAP_GROUP']
    answerMsg = get_all_entries(database_name=k8sGroup,table_name='role')

    if('ErrorCode' in answerMsg):

        typer.echo("Database Error!",err=True)
        typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
        raise typer.Exit(code=1)
    
    roles = list(answerMsg['Enteries'])

    if(name is None):
        if(verbose == 0):
            normalView = []
            for role in roles:
                tempList = []
                tempList.append(role['role_name'])
                tempList.append(role['location'])
                tempList.append(role['version'])
                normalView.append(tempList)
            typer.echo(tabulate(normalView,tablefmt="pretty",headers=["Role Name","Location","Version"]))
        if(verbose > 0):
            extendedView = []
            for role in roles:
                tempList = []
                tempList.append(role['_id'])
                tempList.append(role['role_name'])
                tempList.append(role['location'])
                tempList.append(role['version'])
                tempList.append(role['var_file'])
                extendedView.append(tempList)
            typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["ID","Role Name","Location","Version","VarFile Path"]))

    else:
        answerMsg = CM_query.query_role_by_roleName(database_name=k8sGroup,table_name='role',role=name)
        if('ErrorCode' in answerMsg):
            typer.echo("Database Error!",err=True)
            typer.echo("Error Message : {0}".format(answerMsg['ErrorMsg']),err=True)
            raise typer.Exit(code=1)
        roles = list(answerMsg['Enteries'])
        if(len(roles)==0):
            typer.echo("Role with the name of '{0}' is not found".format(name),err=True)
            raise typer.Exit(code=1)
        else:
            if(verbose == 0):
                normalView = []
                for role in roles:
                    tempList = []
                    tempList.append(role['role_name'])
                    tempList.append(role['location'])
                    tempList.append(role['version'])
                    normalView.append(tempList)
                typer.echo(tabulate(normalView,tablefmt="pretty",headers=["Role Name","Location","Version"]))
            if(verbose > 0):
                extendedView = []
                for role in roles:
                    tempList = []
                    tempList.append(role['_id'])
                    tempList.append(role['role_name'])
                    tempList.append(role['location'])
                    tempList.append(role['version'])
                    tempList.append(role['var_file'])
                    extendedView.append(tempList)
                typer.echo(tabulate(extendedView,tablefmt="pretty",headers=["ID","Role Name","Location","Version","VarFile Path"]))










import hashlib

def get_sha2(attr):
    return hashlib.sha224(attr.encode('utf-8')).hexdigest()

def get_domain_part_from_dn(dn):
    
    list_St = dn.split(',')
    dc = []
    for s in list_St:
        if(s.startswith('DC=')):
            t = s.split('=')[1]
            dc.append(t)

    return '-'.join(dc)


import ChristisRequestor.group as CR_group
import ChristisRequestor.user as CR_user

def get_user_cn(userDN):

    answerMsg = CR_user.get_user_cn(userDN=userDN)
    if ('ErrorCode' in answerMsg):
        return None
    cn = answerMsg['Response']
    return cn



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['christis',
 'christis.cmd_config',
 'christis.cmd_database',
 'christis.cmd_user',
 'christis.engine',
 'christis.engine.core',
 'christis.engine.insert',
 'christis.engine.process',
 'christis.utils']

package_data = \
{'': ['*']}

install_requires = \
['ChristisMongo>=0.21,<0.22',
 'ChristisRequestor>=0.12,<0.13',
 'PyYAML>=5.3.1,<6.0.0',
 'artifacts-keyring>=0.3.1,<0.4.0',
 'keyring>=21.8.0,<22.0.0',
 'pathlib>=1.0.1,<2.0.0',
 'pymongo>=3.11.2,<4.0.0',
 'requests>=2.25.1,<3.0.0',
 'tabulate>=0.8.7,<0.9.0',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['christis = christis.main:app']}

setup_kwargs = {
    'name': 'christis',
    'version': '0.1.0',
    'description': '',
    'long_description': 'Christis is a CLI tool that helps to manage users on the Kubernetes cluster.',
    'author': 'armin',
    'author_email': 'rmin.aminian@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import os

def get_christis_config_location():
    home = os.getenv("HOME")
    return "{0}{1}".format(home,"/.christisCLI/")

def get_mongo_configuration_location():
    return "{0}{1}".format(get_christis_config_location(),"mongo.yaml")

def get_christis_api_config_location():
    return "{0}{1}".format(get_christis_config_location(),"ldap.yaml")

def get_christis_cli_config_location():
    return "{0}{1}".format(get_christis_config_location(),"cli.yaml")


from click.exceptions import ClickException
import typer
from christis.vars import *
from pathlib import Path

app = typer.Typer()


@app.command()
def cli():
    """
    View ChristisCLI Configuration
    """
    cliConfigPath = get_christis_cli_config_location()

    if (not Path(cliConfigPath).is_file()):
        typer.echo("ERROR: The cli.yaml file can't be found please use CLI to generate it!",err=True)
        raise typer.Exit(code=1)
    
    typer.echo("Open CLI Configuration File in {0}".format(cliConfigPath))
    typer.echo("\n")
    with open(cliConfigPath) as cli:
    # Here f is the file-like object
        read_data = cli.read()
        typer.echo(read_data)

@app.command()
def christis_API():
    """
    View ChristisAPI Configuration
    """
    christisAPIconfigPath = get_christis_api_config_location()

    if (not Path(christisAPIconfigPath).is_file()):
        typer.echo("ERROR: The cli.yaml file can't be found please use CLI to generate it!",err=True)
        raise typer.Exit(code=1)
    
    typer.echo("Open CLI Configuration File in {0}".format(christisAPIconfigPath))
    typer.echo("\n")
    with open(christisAPIconfigPath) as api:
    # Here f is the file-like object
        read_data = api.read()
        typer.echo(read_data)

@app.command()
def database():
    """
    View MongoDB Configuration
    """
    christisMongoconfigPath = get_mongo_configuration_location()

    if (not Path(christisMongoconfigPath).is_file()):
        typer.echo("ERROR: The cli.yaml file can't be found please use CLI to generate it!",err=True)
        raise typer.Exit(code=1)
    
    typer.echo("Open CLI Configuration File in {0}".format(christisMongoconfigPath))
    typer.echo("\n")
    with open(christisMongoconfigPath) as mongo:
    # Here f is the file-like object
        read_data = mongo.read()
        typer.echo(read_data)


import ChristisMongo.table.add as CM_add

def add_2_user_table(userList: list,group):
    userCandidateForStage = []
    for user in userList:
      result = CM_add.add_entry_one(database_name=group,table_name="user",data=user)
      if (type(result) == dict and 'ErrorCode' in result):
        if(result['ErrorCode'] != '701'):
            print("Error: MongoDB. Error Message: {0}".format(result['ErrorMsg']))
        continue
      userCandidateForStage.append(user['user_dn'])
    return userCandidateForStage

def add_2_pre_user_table(userList: list,group):
    pre_user = []
    for user in userList:
      result = CM_add.add_entry_one(database_name=group,table_name="pre_user",data=user)
      if (type(result) == dict and 'ErrorCode' in result):
        if(result['ErrorCode'] != '701'):
            print("Error: MongoDB. Error Message: {0}".format(result['ErrorMsg']))
        continue
      pre_user.append(user['user_dn'])
    return pre_user


import typer
import christis.cmd_user.role as role
import christis.cmd_user.view as view

app = typer.Typer()
app.add_typer(role.app,name="role",help="Commands related to assign,unassign,and view user role")
app.add_typer(view.app,name="view",help="Commands relate to view user and its information")


import typer
from christis.vars import *
import christis.config as config
import christis.database as database
import christis.user as user
from christis.utils.general import set_env
from pathlib import Path
from christis.utils.config_parser import get_configuration
import os 
import ChristisRequestor.user as CR_user
import christis.role as role

app = typer.Typer()
app.add_typer(config.app,name="config",help="Commands related to configuration generator")
app.add_typer(database.app,name="database",help="Commands relate to databsae sync and view tables")
app.add_typer(user.app,name="user",help="Commands related to user and its roles")
app.add_typer(role.app,name="role",help="Commands related to role")

@app.callback()
def main(ctx: typer.Context):
    """
    Christis CLI \n
    A Command-Line tool to manage users on Kubernetes cluster
    """
    if (ctx.invoked_subcommand == "config"):
        return
    
    mongoConfigPath = get_mongo_configuration_location()
    cliConfigPath = get_christis_cli_config_location()
    if (not Path(mongoConfigPath).is_file()):
        typer.echo("ERROR: The mongo.yaml file can't be found please use CLI to generate it!",err=True)
        raise typer.Exit(code=1)

    if (not Path(cliConfigPath).is_file()):
        typer.echo("ERROR: The cli.yaml file can't be found please use CLI to generate it!",err=True)
        raise typer.Exit(code=1)
    
    cliConfiguration = get_configuration(cliConfigPath)
    if('ErrorCode' in cliConfiguration):
        typer.echo("ERROR: ldap.yaml",err=True)
        typer.echo(cliConfiguration['ErrorMsg'],err=True)
        typer.Abort()
        raise typer.Exit(code=1)
    
    set_env(cliConfiguration['ChristisCLI'])

app()